@@ -127,6 +127,16 @@ namespace xsimd
     template <class T1, class T2 = T1>
     void store_aligned(T1* dst, const simd_type<T2>& src);
 
+    /**
+     * @ingroup data_transfer
+     * Stores the boolean batch \c src into  the memory array pointed to by \c dst.
+     * \c dst is required to be aligned.
+     * @param dst the pointer to the memory array.
+     * @param src the boolean batch to store.
+     */
+    template <class T1, class T2 = T1>
+    void store_aligned(T1* dst, const simd_bool_type<T2>& src);
+
     /**
      * @ingroup data_transfer
      * Stores the batch \c src into  the memory array pointed to by \c dst.
@@ -137,6 +147,16 @@ namespace xsimd
     template <class T1, class T2 = T1>
     void store_unaligned(T1* dst, const simd_type<T2>& src);
 
+    /**
+     * @ingroup data_transfer
+     * Stores the boolean batch \c src into  the memory array pointed to by \c dst.
+     * \c dst is not required to be aligned.
+     * @param dst the pointer to the memory array.
+     * @param src the boolean batch to store.
+     */
+    template <class T1, class T2 = T1>
+    void store_unaligned(T1* dst, const simd_bool_type<T2>& src);
+
     /**
      * @ingroup data_transfer
      * Stores the batch of complex numbers \c src into  the memory arrays pointed
@@ -263,6 +283,16 @@ namespace xsimd
     template <class T1, class T2 = T1>
     void store_simd(T1* dst, const simd_type<T2>& src, aligned_mode);
 
+    /**
+     * @ingroup generic_load_store
+     * Stores the boolean batch \c src into  the memory array pointed to by \c dst.
+     * \c dst is required to be aligned.
+     * @param dst the pointer to the memory array.
+     * @param src the boolean batch to store.
+     */
+    template <class T1, class T2 = T1>
+    void store_simd(T1* dst, const simd_bool_type<T2>& src, aligned_mode);
+
     /**
      * @ingroup generic_load_store
      * Stores the batch \c src into  the memory array pointed to by \c dst.
@@ -273,6 +303,16 @@ namespace xsimd
     template <class T1, class T2 = T1>
     void store_simd(T1* dst, const simd_type<T2>& src, unaligned_mode);
 
+    /**
+     * @ingroup generic_load_store
+     * Stores the boolean batch \c src into  the memory array pointed to by \c dst.
+     * \c dst is not required to be aligned.
+     * @param dst the pointer to the memory array.
+     * @param src the boolean batch to store.
+     */
+    template <class T1, class T2 = T1>
+    void store_simd(T1* dst, const simd_bool_type<T2>& src, unaligned_mode);
+
     /**
      * @ingroup generic_load_store
      * Stores the batch of complex numbers \c src into  the memory arrays pointed
@@ -495,12 +535,24 @@ namespace xsimd
         detail::simd_function_invoker<T1, simd_type<T2>>::store_aligned(dst, src);
     }
 
+    template <class T1, class T2>
+    inline void store_aligned(T1* dst, const simd_bool_type<T2>& src)
+    {
+        detail::simd_function_invoker<T1, simd_bool_type<T2>>::store_aligned(dst, src);
+    }
+
     template <class T1, class T2>
     inline void store_unaligned(T1* dst, const simd_type<T2>& src)
     {
         detail::simd_function_invoker<T1, simd_type<T2>>::store_unaligned(dst, src);
     }
 
+    template <class T1, class T2>
+    inline void store_unaligned(T1* dst, const simd_bool_type<T2>& src)
+    {
+        detail::simd_function_invoker<T1, simd_bool_type<T2>>::store_unaligned(dst, src);
+    }
+
     template <class T1, class T2>
     inline void store_aligned(T1* real_dst, T1* imag_dst, const simd_type<T2>& src)
     {
@@ -571,12 +623,24 @@ namespace xsimd
         store_aligned<T1, T2>(dst, src);
     }
 
+    template <class T1, class T2>
+    inline void store_simd(T1* dst, const simd_bool_type<T2>& src, aligned_mode)
+    {
+        store_aligned<T1, T2>(dst, src);
+    }
+
     template <class T1, class T2>
     inline void store_simd(T1* dst, const simd_type<T2>& src, unaligned_mode)
     {
         store_unaligned<T1, T2>(dst, src);
     }
 
+    template <class T1, class T2>
+    inline void store_simd(T1* dst, const simd_bool_type<T2>& src, unaligned_mode)
+    {
+        store_unaligned<T1, T2>(dst, src);
+    }
+
     template <class T1, class T2>
     inline void store_simd(T1* real_dst, T1* imag_dst, const simd_type<T2>& src, aligned_mode)
     {
