@@ -31,8 +31,7 @@ namespace xsimd
     };
 
     template <>
-    class batch_bool<float, 16> : public batch_bool_avx512<__mmask16, batch_bool<float, 16>>,
-                                  public simd_batch_bool<batch_bool<float, 16>>
+    class batch_bool<float, 16> : public batch_bool_avx512<__mmask16, batch_bool<float, 16>>
     {
     public:
         using base_class = batch_bool_avx512<__mmask16, batch_bool<float, 16>>;
@@ -59,6 +58,7 @@ namespace xsimd
         using batch_bool_type = batch_bool<float, 16>;
         static constexpr std::size_t size = 16;
         static constexpr std::size_t align = 64;
+        using storage_type = __m512;
     };
 
     template <>
@@ -79,20 +79,18 @@ namespace xsimd
         batch(const __m512& rhs);
         batch& operator=(const __m512& rhs);
 
+        batch(const batch_bool<float, 16>& rhs);
+        batch& operator=(const batch_bool<float, 16>& rhs);
+
         operator __m512() const;
 
-        XSIMD_DECLARE_LOAD_STORE_ALL(float, 16);
+        XSIMD_DECLARE_LOAD_STORE_ALL(float, 16)
+        XSIMD_DECLARE_LOAD_STORE_LONG(float, 16)
 
         using base_type::load_aligned;
         using base_type::load_unaligned;
         using base_type::store_aligned;
         using base_type::store_unaligned;
-
-        float operator[](std::size_t index) const;
-
-    private:
-
-        __m512 m_value;
     };
 
     /************************************
@@ -104,52 +102,52 @@ namespace xsimd
     }
 
     inline batch<float, 16>::batch(float i)
-        : m_value(_mm512_set1_ps(i))
+        : base_type(_mm512_set1_ps(i))
     {
     }
 
     inline batch<float, 16>::batch(float i0, float i1,  float i2,  float i3,  float i4,  float i5,  float i6,  float i7,
                                    float i8, float i9, float i10, float i11, float i12, float i13, float i14, float i15)
-        : m_value(_mm512_setr_ps(i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, i10, i11, i12, i13, i14, i15))
+        : base_type(_mm512_setr_ps(i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, i10, i11, i12, i13, i14, i15))
     {
     }
 
     inline batch<float, 16>::batch(const float* src)
-        : m_value(_mm512_loadu_ps(src))
+        : base_type(_mm512_loadu_ps(src))
     {
     }
 
     inline batch<float, 16>::batch(const float* src, aligned_mode)
-        : m_value(_mm512_load_ps(src))
+        : base_type(_mm512_load_ps(src))
     {
     }
 
     inline batch<float, 16>::batch(const float* src, unaligned_mode)
-        : m_value(_mm512_loadu_ps(src))
+        : base_type(_mm512_loadu_ps(src))
     {
     }
 
     inline batch<float, 16>::batch(const __m512& rhs)
-        : m_value(rhs)
+        : base_type(rhs)
     {
     }
 
     inline batch<float, 16>& batch<float, 16>::operator=(const __m512& rhs)
     {
-        m_value = rhs;
+        this->m_value = rhs;
         return *this;
     }
 
     inline batch<float, 16>::operator __m512() const
     {
-        return m_value;
+        return this->m_value;
     }
 
     inline batch<float, 16>& batch<float, 16>::load_aligned(const int8_t* src)
     {
         __m128i tmp = _mm_load_si128((const __m128i*)src);
         __m512i tmp2 = _mm512_cvtepi8_epi32(tmp);
-        m_value = _mm512_cvtepi32_ps(tmp2);
+        this->m_value = _mm512_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -157,7 +155,7 @@ namespace xsimd
     {
         __m128i tmp = _mm_loadu_si128((const __m128i*)src);
         __m512i tmp2 = _mm512_cvtepi8_epi32(tmp);
-        m_value = _mm512_cvtepi32_ps(tmp2);
+        this->m_value = _mm512_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -165,7 +163,7 @@ namespace xsimd
     {
         __m128i tmp = _mm_load_si128((const __m128i*)src);
         __m512i tmp2 = _mm512_cvtepu8_epi32(tmp);
-        m_value = _mm512_cvtepi32_ps(tmp2);
+        this->m_value = _mm512_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -173,7 +171,7 @@ namespace xsimd
     {
         __m128i tmp = _mm_loadu_si128((const __m128i*)src);
         __m512i tmp2 = _mm512_cvtepu8_epi32(tmp);
-        m_value = _mm512_cvtepi32_ps(tmp2);
+        this->m_value = _mm512_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -181,7 +179,7 @@ namespace xsimd
     {
         __m256i tmp = _mm256_load_si256((const __m256i*)src);
         __m512i tmp2 = _mm512_cvtepi16_epi32(tmp);
-        m_value = _mm512_cvtepi32_ps(tmp2);
+        this->m_value = _mm512_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -189,7 +187,7 @@ namespace xsimd
     {
         __m256i tmp = _mm256_loadu_si256((const __m256i*)src);
         __m512i tmp2 = _mm512_cvtepi16_epi32(tmp);
-        m_value = _mm512_cvtepi32_ps(tmp2);
+        this->m_value = _mm512_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -197,7 +195,7 @@ namespace xsimd
     {
         __m256i tmp = _mm256_load_si256((const __m256i*)src);
         __m512i tmp2 = _mm512_cvtepu16_epi32(tmp);
-        m_value = _mm512_cvtepi32_ps(tmp2);
+        this->m_value = _mm512_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -205,48 +203,49 @@ namespace xsimd
     {
         __m256i tmp = _mm256_loadu_si256((const __m256i*)src);
         __m512i tmp2 = _mm512_cvtepu16_epi32(tmp);
-        m_value = _mm512_cvtepi32_ps(tmp2);
+        this->m_value = _mm512_cvtepi32_ps(tmp2);
         return *this;
     }
 
     inline batch<float, 16>& batch<float, 16>::load_aligned(const int32_t* src)
     {
         // TODO select correct rounding direction?
-        m_value = _mm512_cvt_roundepi32_ps(_mm512_load_si512(src), _MM_FROUND_CUR_DIRECTION);
+        this->m_value = _mm512_cvt_roundepi32_ps(_mm512_load_si512(src), _MM_FROUND_CUR_DIRECTION);
         return *this;
     }
 
     inline batch<float, 16>& batch<float, 16>::load_unaligned(const int32_t* src)
     {
-        m_value = _mm512_cvt_roundepi32_ps(_mm512_loadu_si512(src), _MM_FROUND_CUR_DIRECTION);
+        this->m_value = _mm512_cvt_roundepi32_ps(_mm512_loadu_si512(src), _MM_FROUND_CUR_DIRECTION);
         return *this;
     }
 
     inline batch<float, 16>& batch<float, 16>::load_aligned(const uint32_t* src)
     {
         // TODO select correct rounding direction?
-        m_value = _mm512_cvt_roundepu32_ps(_mm512_load_si512(src), _MM_FROUND_CUR_DIRECTION);
+        this->m_value = _mm512_cvt_roundepu32_ps(_mm512_load_si512(src), _MM_FROUND_CUR_DIRECTION);
         return *this;
     }
 
     inline batch<float, 16>& batch<float, 16>::load_unaligned(const uint32_t* src)
     {
-        m_value = _mm512_cvt_roundepu32_ps(_mm512_loadu_si512(src), _MM_FROUND_CUR_DIRECTION);
+        this->m_value = _mm512_cvt_roundepu32_ps(_mm512_loadu_si512(src), _MM_FROUND_CUR_DIRECTION);
         return *this;
     }
 
     XSIMD_DEFINE_LOAD_STORE(float, 16, int64_t, 64)
     XSIMD_DEFINE_LOAD_STORE(float, 16, uint64_t, 64)
+    XSIMD_DEFINE_LOAD_STORE_LONG(float, 16, 64)
 
     inline batch<float, 16>& batch<float, 16>::load_aligned(const float* src)
     {
-        m_value = _mm512_load_ps(src);
+        this->m_value = _mm512_load_ps(src);
         return *this;
     }
 
     inline batch<float, 16>& batch<float, 16>::load_unaligned(const float* src)
     {
-        m_value = _mm512_loadu_ps(src);
+        this->m_value = _mm512_loadu_ps(src);
         return *this;
     }
 
@@ -254,8 +253,8 @@ namespace xsimd
     {
         __m256 tmp1 = _mm512_cvtpd_ps(_mm512_load_pd(src));
         __m256 tmp2 = _mm512_cvtpd_ps(_mm512_load_pd(src + 8));
-        m_value = _mm512_castps256_ps512(tmp1);
-        m_value = _mm512_insertf32x8(m_value, tmp2, 1);
+        this->m_value = _mm512_castps256_ps512(tmp1);
+        this->m_value = _mm512_insertf32x8(this->m_value, tmp2, 1);
         return *this;
     }
 
@@ -263,114 +262,107 @@ namespace xsimd
     {
         __m256 tmp1 = _mm512_cvtpd_ps(_mm512_loadu_pd(src));
         __m256 tmp2 = _mm512_cvtpd_ps(_mm512_loadu_pd(src + 8));
-        m_value = _mm512_castps256_ps512(tmp1);
-        m_value = _mm512_insertf32x8(m_value, tmp2, 1);
+        this->m_value = _mm512_castps256_ps512(tmp1);
+        this->m_value = _mm512_insertf32x8(this->m_value, tmp2, 1);
         return *this;
     }
 
     inline void batch<float, 16>::store_aligned(int8_t* dst) const
     {
-        __m512i tmp = _mm512_cvtps_epi32(m_value);
+        __m512i tmp = _mm512_cvtps_epi32(this->m_value);
         __m128i tmp2 = _mm512_cvtepi32_epi8(tmp);
         _mm_store_si128((__m128i*)dst, tmp2);
     }
 
     inline void batch<float, 16>::store_unaligned(int8_t* dst) const
     {
-        __m512i tmp = _mm512_cvtps_epi32(m_value);
+        __m512i tmp = _mm512_cvtps_epi32(this->m_value);
         __m128i tmp2 = _mm512_cvtepi32_epi8(tmp);
         _mm_storeu_si128((__m128i*)dst, tmp2);
     }
 
     inline void batch<float, 16>::store_aligned(uint8_t* dst) const
     {
-        __m512i tmp = _mm512_cvtps_epu32(m_value);
+        __m512i tmp = _mm512_cvtps_epu32(this->m_value);
         __m128i tmp2 = _mm512_cvtusepi32_epi8(tmp);
         _mm_store_si128((__m128i*)dst, tmp2);
     }
 
     inline void batch<float, 16>::store_unaligned(uint8_t* dst) const
     {
-        __m512i tmp = _mm512_cvtps_epu32(m_value);
+        __m512i tmp = _mm512_cvtps_epu32(this->m_value);
         __m128i tmp2 = _mm512_cvtusepi32_epi8(tmp);
         _mm_storeu_si128((__m128i*)dst, tmp2);
     }
 
     inline void batch<float, 16>::store_aligned(int16_t* dst) const
     {
-        __m512i tmp = _mm512_cvtps_epi32(m_value);
+        __m512i tmp = _mm512_cvtps_epi32(this->m_value);
         __m256i tmp2 = _mm512_cvtepi32_epi16(tmp);
         _mm256_store_si256((__m256i*)dst, tmp2);
     }
 
     inline void batch<float, 16>::store_unaligned(int16_t* dst) const
     {
-        __m512i tmp = _mm512_cvtps_epi32(m_value);
+        __m512i tmp = _mm512_cvtps_epi32(this->m_value);
         __m256i tmp2 = _mm512_cvtepi32_epi16(tmp);
         _mm256_storeu_si256((__m256i*)dst, tmp2);
     }
 
     inline void batch<float, 16>::store_aligned(uint16_t* dst) const
     {
-        __m512i tmp = _mm512_cvtps_epu32(m_value);
+        __m512i tmp = _mm512_cvtps_epu32(this->m_value);
         __m256i tmp2 = _mm512_cvtusepi32_epi16(tmp);
         _mm256_store_si256((__m256i*)dst, tmp2);
     }
 
     inline void batch<float, 16>::store_unaligned(uint16_t* dst) const
     {
-        __m512i tmp = _mm512_cvtps_epu32(m_value);
+        __m512i tmp = _mm512_cvtps_epu32(this->m_value);
         __m256i tmp2 = _mm512_cvtusepi32_epi16(tmp);
         _mm256_storeu_si256((__m256i*)dst, tmp2);
     }
 
     inline void batch<float, 16>::store_aligned(int32_t* dst) const
     {
-        _mm512_store_si512((__m512i *)dst, _mm512_cvtps_epi32(m_value));
+        _mm512_store_si512((__m512i *)dst, _mm512_cvtps_epi32(this->m_value));
     }
 
     inline void batch<float, 16>::store_unaligned(int32_t* dst) const
     {
-        _mm512_storeu_si512((__m512i *)dst, _mm512_cvtps_epi32(m_value));
+        _mm512_storeu_si512((__m512i *)dst, _mm512_cvtps_epi32(this->m_value));
     }
 
     inline void batch<float, 16>::store_aligned(uint32_t* dst) const
     {
-        _mm512_store_si512((__m512i *)dst, _mm512_cvtps_epu32(m_value));
+        _mm512_store_si512((__m512i *)dst, _mm512_cvtps_epu32(this->m_value));
     }
 
     inline void batch<float, 16>::store_unaligned(uint32_t* dst) const
     {
-        _mm512_storeu_si512((__m512i *)dst, _mm512_cvtps_epu32(m_value));
+        _mm512_storeu_si512((__m512i *)dst, _mm512_cvtps_epu32(this->m_value));
     }
 
     inline void batch<float, 16>::store_aligned(float* dst) const
     {
-        _mm512_store_ps(dst, m_value);
+        _mm512_store_ps(dst, this->m_value);
     }
 
     inline void batch<float, 16>::store_unaligned(float* dst) const
     {
-        _mm512_storeu_ps(dst, m_value);
+        _mm512_storeu_ps(dst, this->m_value);
     }
 
     inline void batch<float, 16>::store_aligned(double* dst) const
     {
-        _mm512_store_pd(dst, _mm512_cvtps_pd(_mm512_extractf32x8_ps(m_value, 0)));
-        _mm512_store_pd(dst + 8, _mm512_cvtps_pd(_mm512_extractf32x8_ps(m_value, 1)));
+        _mm512_store_pd(dst, _mm512_cvtps_pd(_mm512_extractf32x8_ps(this->m_value, 0)));
+        _mm512_store_pd(dst + 8, _mm512_cvtps_pd(_mm512_extractf32x8_ps(this->m_value, 1)));
     }
 
     inline void batch<float, 16>::store_unaligned(double* dst) const
     {
-        _mm512_storeu_pd(dst, _mm512_cvtps_pd(_mm512_extractf32x8_ps(m_value, 0)));
-        _mm512_storeu_pd(dst + 8, _mm512_cvtps_pd(_mm512_extractf32x8_ps(m_value, 1)));
-    }
-
-    inline float batch<float, 16>::operator[](std::size_t index) const
-    {
-        alignas(64) float x[16];
-        store_aligned(x);
-        return x[index & 15];
+        _mm512_storeu_pd(dst, _mm512_cvtps_pd(_mm512_extractf32x8_ps(this->m_value, 0)));
+        _mm512_storeu_pd(dst + 8, _mm512_cvtps_pd(_mm512_extractf32x8_ps(this->m_value, 1)));
     }
 
     namespace detail
@@ -516,7 +508,7 @@ namespace xsimd
                 return xsimd::hadd(batch<float, 8>(res1));
             }
 
-            static batch_type haddp(const simd_batch<batch_type>* row)
+            static batch_type haddp(const batch_type* row)
             {
                 // The following folds over the vector once:
                 // tmp1 = [a0..8, b0..8]
@@ -529,14 +521,14 @@ namespace xsimd
             res ## I = tmp1 + tmp2;                                                            \
         }                                                                                      \
 
-                XSIMD_AVX512_HADDP_STEP1(1, row[0](), row[4]());
-                XSIMD_AVX512_HADDP_STEP1(2, row[2](), row[6]());
-                XSIMD_AVX512_HADDP_STEP1(3, row[1](), row[5]());
-                XSIMD_AVX512_HADDP_STEP1(4, row[3](), row[7]());
-                XSIMD_AVX512_HADDP_STEP1(5, row[8](), row[12]());
-                XSIMD_AVX512_HADDP_STEP1(6, row[10](), row[14]());
-                XSIMD_AVX512_HADDP_STEP1(7, row[9](), row[13]());
-                XSIMD_AVX512_HADDP_STEP1(8, row[11](), row[15]());
+                XSIMD_AVX512_HADDP_STEP1(0, row[0], row[2]);
+                XSIMD_AVX512_HADDP_STEP1(1, row[4], row[6]);
+                XSIMD_AVX512_HADDP_STEP1(2, row[1], row[3]);
+                XSIMD_AVX512_HADDP_STEP1(3, row[5], row[7]);
+                XSIMD_AVX512_HADDP_STEP1(4, row[8], row[10]);
+                XSIMD_AVX512_HADDP_STEP1(5, row[12], row[14]);
+                XSIMD_AVX512_HADDP_STEP1(6, row[9], row[11]);
+                XSIMD_AVX512_HADDP_STEP1(7, row[13], row[15]);
 
 #undef XSIMD_AVX512_HADDP_STEP1
 
@@ -557,8 +549,8 @@ namespace xsimd
                                                                                                 \
             batch<float, 16> resx2 = tmp3 + tmp4;                                               \
                                                                                                 \
-            batch<float, 16> tmp5 = _mm512_shuffle_ps(resx1, resx2, 0b00000000);                \
-            batch<float, 16> tmp6 = _mm512_shuffle_ps(resx1, resx2, 0b11111111);                \
+            batch<float, 16> tmp5 = _mm512_shuffle_ps(resx1, resx2, _MM_SHUFFLE(2, 0, 2, 0));   \
+            batch<float, 16> tmp6 = _mm512_shuffle_ps(resx1, resx2, _MM_SHUFFLE(3, 1, 3, 1));   \
                                                                                                 \
             batch<float, 16> resx3 = tmp5 + tmp6;                                               \
                                                                                                 \
@@ -566,8 +558,8 @@ namespace xsimd
                                          _mm512_extractf32x8_ps(resx3, 1));                     \
         }                                                                                       \
 
-                XSIMD_AVX512_HADDP_STEP2(0, res1, res2, res3, res4);
-                XSIMD_AVX512_HADDP_STEP2(1, res5, res6, res7, res8);
+                XSIMD_AVX512_HADDP_STEP2(0, res0, res1, res2, res3);
+                XSIMD_AVX512_HADDP_STEP2(1, res4, res5, res6, res7);
 
 #undef XSIMD_AVX512_HADDP_STEP2
 
@@ -587,6 +579,18 @@ namespace xsimd
             }
         };
     }
+
+    inline batch<float, 16>::batch(const batch_bool<float, 16>& rhs)
+        : base_type(detail::batch_kernel<float, 16>::select(rhs, batch(float(1)), batch(float(0))))
+    {
+    }
+
+    inline batch<float, 16>& batch<float, 16>::operator=(const batch_bool<float, 16>& rhs)
+    {
+        this->m_value = detail::batch_kernel<float, 16>::select(rhs, batch(float(1)), batch(float(0)));
+        return *this;
+    }
+
 }
 
 #endif
