@@ -44,8 +44,7 @@ namespace xsimd
 
     template <>
     class batch_bool<int16_t, 32> :
-        public batch_bool_avx512<__mmask32, batch_bool<int16_t, 32>>,
-        public simd_batch_bool<batch_bool<int16_t, 32>>
+        public batch_bool_avx512<__mmask32, batch_bool<int16_t, 32>>
     {
     public:
 
@@ -55,8 +54,7 @@ namespace xsimd
 
     template <>
     class batch_bool<uint16_t, 32> :
-        public batch_bool_avx512<__mmask32, batch_bool<uint16_t, 32>>,
-        public simd_batch_bool<batch_bool<uint16_t, 32>>
+        public batch_bool_avx512<__mmask32, batch_bool<uint16_t, 32>>
     {
     public:
 
@@ -128,6 +126,7 @@ namespace xsimd
         static constexpr std::size_t size = 32;
         using batch_bool_type = batch_bool<int16_t, 32>;
         static constexpr std::size_t align = 64;
+        using storage_type = __m512i;
     };
 
     template <>
@@ -137,6 +136,7 @@ namespace xsimd
         static constexpr std::size_t size = 32;
         using batch_bool_type = batch_bool<uint16_t, 32>;
         static constexpr std::size_t align = 64;
+        using storage_type = __m512i;
     };
 
     template <>
@@ -168,7 +168,8 @@ namespace xsimd
         {
         }
 
-        XSIMD_DECLARE_LOAD_STORE_INT16(int16_t, 32);
+        XSIMD_DECLARE_LOAD_STORE_INT16(int16_t, 32)
+        XSIMD_DECLARE_LOAD_STORE_LONG(int16_t, 32)
     };
 
     template <>
@@ -183,7 +184,8 @@ namespace xsimd
         using base_class::store_aligned;
         using base_class::store_unaligned;
 
-        XSIMD_DECLARE_LOAD_STORE_INT16(uint16_t, 32);
+        XSIMD_DECLARE_LOAD_STORE_INT16(uint16_t, 32)
+        XSIMD_DECLARE_LOAD_STORE_LONG(uint16_t, 32)
     };
 
     batch<int16_t, 32> operator<<(const batch<int16_t, 32>& lhs, int32_t rhs);
@@ -466,6 +468,7 @@ namespace xsimd
     }
 
     XSIMD_DEFINE_LOAD_STORE_INT16(int16_t, 32, 32)
+    XSIMD_DEFINE_LOAD_STORE_LONG(int16_t, 32, 32)
 
     inline batch<uint16_t, 32> operator<<(const batch<uint16_t, 32>& lhs, int32_t rhs)
     {
@@ -482,6 +485,7 @@ namespace xsimd
     }
 
     XSIMD_DEFINE_LOAD_STORE_INT16(uint16_t, 32, 32)
+    XSIMD_DEFINE_LOAD_STORE_LONG(uint16_t, 32, 32)
 
 #undef XSIMD_APPLY_AVX2_FUNCTION_INT16
 }
