@@ -44,8 +44,7 @@ namespace xsimd
 
     template <>
     class batch_bool<int8_t, 64> :
-        public batch_bool_avx512<__mmask64, batch_bool<int8_t, 64>>,
-        public simd_batch_bool<batch_bool<int8_t, 64>>
+        public batch_bool_avx512<__mmask64, batch_bool<int8_t, 64>>
     {
     public:
 
@@ -55,8 +54,7 @@ namespace xsimd
 
     template <>
     class batch_bool<uint8_t, 64> :
-        public batch_bool_avx512<__mmask64, batch_bool<uint8_t, 64>>,
-        public simd_batch_bool<batch_bool<uint8_t, 64>>
+        public batch_bool_avx512<__mmask64, batch_bool<uint8_t, 64>>
     {
     public:
 
@@ -128,6 +126,7 @@ namespace xsimd
         static constexpr std::size_t size = 64;
         using batch_bool_type = batch_bool<int8_t, 64>;
         static constexpr std::size_t align = 64;
+        using storage_type = __m512i;
     };
 
     template <>
@@ -137,6 +136,7 @@ namespace xsimd
         static constexpr std::size_t size = 64;
         using batch_bool_type = batch_bool<uint8_t, 64>;
         static constexpr std::size_t align = 64;
+        using storage_type = __m512i;
     };
 
     template <>
@@ -168,7 +168,8 @@ namespace xsimd
         {
         }
 
-        XSIMD_DECLARE_LOAD_STORE_INT8(int8_t, 64);
+        XSIMD_DECLARE_LOAD_STORE_INT8(int8_t, 64)
+        XSIMD_DECLARE_LOAD_STORE_LONG(int8_t, 64)
     };
 
     template <>
@@ -183,7 +184,8 @@ namespace xsimd
         using base_class::store_aligned;
         using base_class::store_unaligned;
 
-        XSIMD_DECLARE_LOAD_STORE_INT8(uint8_t, 64);
+        XSIMD_DECLARE_LOAD_STORE_INT8(uint8_t, 64)
+        XSIMD_DECLARE_LOAD_STORE_LONG(uint8_t, 64)
     };
 
     batch<int8_t, 64> operator<<(const batch<int8_t, 64>& lhs, int32_t rhs);
@@ -468,6 +470,7 @@ namespace xsimd
     }
 
     XSIMD_DEFINE_LOAD_STORE_INT8(int8_t, 64, 64)
+    XSIMD_DEFINE_LOAD_STORE_LONG(int8_t, 64, 64)
 
     inline batch<uint8_t, 64> operator<<(const batch<uint8_t, 64>& lhs, int32_t rhs)
     {
@@ -484,6 +487,7 @@ namespace xsimd
     }
 
     XSIMD_DEFINE_LOAD_STORE_INT8(uint8_t, 64, 64)
+    XSIMD_DEFINE_LOAD_STORE_LONG(uint8_t, 64, 64)
 
 #undef XSIMD_APPLY_AVX2_FUNCTION_INT8
 }
