@@ -11,6 +11,7 @@
 
 #include "xsimd_base.hpp"
 #include "xsimd_int_conversion.hpp"
+#include <array>
 
 namespace xsimd
 {
@@ -42,11 +43,23 @@ namespace xsimd
 
         operator __m256() const;
 
+        bool_proxy<float> operator[](std::size_t index);
         bool operator[](std::size_t index) const;
 
+        __m256 get_value() const;
+
     private:
 
-        __m256 m_value;
+        batch_bool<float, 8>& load_values(bool b0, bool b1, bool b2, bool b3,
+                                          bool b4, bool b5, bool b6, bool b7);
+
+        union
+        {
+            __m256 m_value;
+            float m_array[8];
+        };
+
+        friend class simd_batch_bool<batch_bool<float, 8>>;
     };
 
     /*******************
@@ -60,6 +73,7 @@ namespace xsimd
         static constexpr std::size_t size = 8;
         using batch_bool_type = batch_bool<float, 8>;
         static constexpr std::size_t align = 32;
+        using storage_type = __m256;
     };
 
     template <>
@@ -82,18 +96,13 @@ namespace xsimd
 
         operator __m256() const;
 
-        XSIMD_DECLARE_LOAD_STORE_ALL(float, 8);
+        XSIMD_DECLARE_LOAD_STORE_ALL(float, 8)
+        XSIMD_DECLARE_LOAD_STORE_LONG(float, 8)
 
         using base_type::load_aligned;
         using base_type::load_unaligned;
         using base_type::store_aligned;
         using base_type::store_unaligned;
-
-        float operator[](std::size_t index) const;
-
-    private:
-
-        __m256 m_value;
     };
 
     /***************************************
@@ -105,21 +114,21 @@ namespace xsimd
     }
 
     inline batch_bool<float, 8>::batch_bool(bool b)
-        : m_value(_mm256_castsi256_ps(_mm256_set1_epi32(-(int)b)))
     {
+        m_value = _mm256_castsi256_ps(_mm256_set1_epi32(-(int)b));
     }
 
     inline batch_bool<float, 8>::batch_bool(bool b0, bool b1, bool b2, bool b3,
                                             bool b4, bool b5, bool b6, bool b7)
-        : m_value(_mm256_castsi256_ps(
-              _mm256_setr_epi32(-(int)b0, -(int)b1, -(int)b2, -(int)b3,
-                                -(int)b4, -(int)b5, -(int)b6, -(int)b7)))
     {
+        m_value = _mm256_castsi256_ps(
+              _mm256_setr_epi32(-(int)b0, -(int)b1, -(int)b2, -(int)b3,
+                                -(int)b4, -(int)b5, -(int)b6, -(int)b7));
     }
 
     inline batch_bool<float, 8>::batch_bool(const __m256& rhs)
-        : m_value(rhs)
     {
+        m_value = rhs;
     }
 
     inline batch_bool<float, 8>& batch_bool<float, 8>::operator=(const __m256& rhs)
@@ -133,11 +142,28 @@ namespace xsimd
         return m_value;
     }
 
+    inline bool_proxy<float> batch_bool<float, 8>::operator[](std::size_t index)
+    {
+        return bool_proxy<float>(m_array[index & 7]);
+    }
+
     inline bool batch_bool<float, 8>::operator[](std::size_t index) const
     {
-        alignas(32) float x[8];
-        _mm256_store_ps(x, m_value);
-        return static_cast<bool>(x[index & 7]);
+        return static_cast<bool>(m_array[index & 7]);
+    }
+
+    inline __m256 batch_bool<float, 8>::get_value() const
+    {
+        return m_value;
+    }
+
+    inline batch_bool<float, 8>& batch_bool<float, 8>::load_values(bool b0, bool b1, bool b2, bool b3,
+                                                                   bool b4, bool b5, bool b6, bool b7)
+    {
+        m_value = _mm256_castsi256_ps(
+              _mm256_setr_epi32(-(int)b0, -(int)b1, -(int)b2, -(int)b3,
+                                -(int)b4, -(int)b5, -(int)b6, -(int)b7));
+        return *this;
     }
 
     namespace detail
@@ -214,52 +240,52 @@ namespace xsimd
     }
 
     inline batch<float, 8>::batch(float f)
-        : m_value(_mm256_set1_ps(f))
+        : base_type(_mm256_set1_ps(f))
     {
     }
 
     inline batch<float, 8>::batch(float f0, float f1, float f2, float f3,
                                   float f4, float f5, float f6, float f7)
-        : m_value(_mm256_setr_ps(f0, f1, f2, f3, f4, f5, f6, f7))
+        : base_type(_mm256_setr_ps(f0, f1, f2, f3, f4, f5, f6, f7))
     {
     }
 
     inline batch<float, 8>::batch(const float* src)
-        : m_value(_mm256_loadu_ps(src))
+        : base_type(_mm256_loadu_ps(src))
     {
     }
 
     inline batch<float, 8>::batch(const float* src, aligned_mode)
-        : m_value(_mm256_load_ps(src))
+        : base_type(_mm256_load_ps(src))
     {
     }
 
     inline batch<float, 8>::batch(const float* src, unaligned_mode)
-        : m_value(_mm256_loadu_ps(src))
+        : base_type(_mm256_loadu_ps(src))
     {
     }
 
     inline batch<float, 8>::batch(const __m256& rhs)
-        : m_value(rhs)
+        : base_type(rhs)
     {
     }
 
     inline batch<float, 8>& batch<float, 8>::operator=(const __m256& rhs)
     {
-        m_value = rhs;
+        this->m_value = rhs;
         return *this;
     }
 
     inline batch<float, 8>::operator __m256() const
     {
-        return m_value;
+        return this->m_value;
     }
 
     inline batch<float, 8>& batch<float, 8>::load_aligned(const int8_t* src)
     {
         __m128i tmp = _mm_loadl_epi64((const __m128i*)src);
         __m256i tmp2 = detail::xsimd_cvtepi8_epi32(tmp);
-        m_value = _mm256_cvtepi32_ps(tmp2);
+        this->m_value = _mm256_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -272,7 +298,7 @@ namespace xsimd
     {
         __m128i tmp = _mm_loadl_epi64((const __m128i*)src);
         __m256i tmp2 = detail::xsimd_cvtepu8_epi32(tmp);
-        m_value = _mm256_cvtepi32_ps(tmp2);
+        this->m_value = _mm256_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -285,7 +311,7 @@ namespace xsimd
     {
         __m128i tmp = _mm_load_si128((const __m128i*)src);
         __m256i tmp2 = detail::xsimd_cvtepi16_epi32(tmp);
-        m_value = _mm256_cvtepi32_ps(tmp2);
+        this->m_value = _mm256_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -293,7 +319,7 @@ namespace xsimd
     {
         __m128i tmp = _mm_loadu_si128((const __m128i*)src);
         __m256i tmp2 = detail::xsimd_cvtepi16_epi32(tmp);
-        m_value = _mm256_cvtepi32_ps(tmp2);
+        this->m_value = _mm256_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -301,7 +327,7 @@ namespace xsimd
     {
         __m128i tmp = _mm_load_si128((const __m128i*)src);
         __m256i tmp2 = detail::xsimd_cvtepu16_epi32(tmp);
-        m_value = _mm256_cvtepi32_ps(tmp2);
+        this->m_value = _mm256_cvtepi32_ps(tmp2);
         return *this;
     }
 
@@ -309,35 +335,36 @@ namespace xsimd
     {
         __m128i tmp = _mm_loadu_si128((const __m128i*)src);
         __m256i tmp2 = detail::xsimd_cvtepu16_epi32(tmp);
-        m_value = _mm256_cvtepi32_ps(tmp2);
+        this->m_value = _mm256_cvtepi32_ps(tmp2);
         return *this;
     }
 
     inline batch<float, 8>& batch<float, 8>::load_aligned(const int32_t* src)
     {
-        m_value = _mm256_cvtepi32_ps(_mm256_load_si256((__m256i const*)src));
+        this->m_value = _mm256_cvtepi32_ps(_mm256_load_si256((__m256i const*)src));
         return *this;
     }
 
     inline batch<float, 8>& batch<float, 8>::load_unaligned(const int32_t* src)
     {
-        m_value = _mm256_cvtepi32_ps(_mm256_loadu_si256((__m256i const*)src));
+        this->m_value = _mm256_cvtepi32_ps(_mm256_loadu_si256((__m256i const*)src));
         return *this;
     }
 
     XSIMD_DEFINE_LOAD_STORE(float, 8, uint32_t, 32)
     XSIMD_DEFINE_LOAD_STORE(float, 8, int64_t, 32)
     XSIMD_DEFINE_LOAD_STORE(float, 8, uint64_t, 32)
+    XSIMD_DEFINE_LOAD_STORE_LONG(float, 8, 32)
 
     inline batch<float, 8>& batch<float, 8>::load_aligned(const float* src)
     {
-        m_value = _mm256_load_ps(src);
+        this->m_value = _mm256_load_ps(src);
         return *this;
     }
 
     inline batch<float, 8>& batch<float, 8>::load_unaligned(const float* src)
     {
-        m_value = _mm256_loadu_ps(src);
+        this->m_value = _mm256_loadu_ps(src);
         return *this;
     }
 
@@ -345,8 +372,8 @@ namespace xsimd
     {
         __m128 tmp1 = _mm256_cvtpd_ps(_mm256_load_pd(src));
         __m128 tmp2 = _mm256_cvtpd_ps(_mm256_load_pd(src + 4));
-        m_value = _mm256_castps128_ps256(tmp1);
-        m_value = _mm256_insertf128_ps(m_value, tmp2, 1);
+        this->m_value = _mm256_castps128_ps256(tmp1);
+        this->m_value = _mm256_insertf128_ps(this->m_value, tmp2, 1);
         return *this;
     }
 
@@ -354,14 +381,14 @@ namespace xsimd
     {
         __m128 tmp1 = _mm256_cvtpd_ps(_mm256_loadu_pd(src));
         __m128 tmp2 = _mm256_cvtpd_ps(_mm256_loadu_pd(src + 4));
-        m_value = _mm256_castps128_ps256(tmp1);
-        m_value = _mm256_insertf128_ps(m_value, tmp2, 1);
+        this->m_value = _mm256_castps128_ps256(tmp1);
+        this->m_value = _mm256_insertf128_ps(this->m_value, tmp2, 1);
         return *this;
     }
 
     inline void batch<float, 8>::store_aligned(int8_t* dst) const
     {
-        __m256i tmp = _mm256_cvtps_epi32(m_value);
+        __m256i tmp = _mm256_cvtps_epi32(this->m_value);
         __m128i tmp2 = detail::xsimd_cvtepi32_epi8(tmp);
         _mm_storel_epi64((__m128i*)dst, tmp2);
     }
@@ -373,7 +400,7 @@ namespace xsimd
 
     inline void batch<float, 8>::store_aligned(uint8_t* dst) const
     {
-        __m256i tmp = _mm256_cvtps_epi32(m_value);
+        __m256i tmp = _mm256_cvtps_epi32(this->m_value);
         __m128i tmp2 = detail::xsimd_cvtepi32_epu8(tmp);
         _mm_storel_epi64((__m128i*)dst, tmp2);
     }
@@ -385,56 +412,56 @@ namespace xsimd
 
     inline void batch<float, 8>::store_aligned(int16_t* dst) const
     {
-        __m256i tmp = _mm256_cvtps_epi32(m_value);
+        __m256i tmp = _mm256_cvtps_epi32(this->m_value);
         __m128i tmp2 = detail::xsimd_cvtepi32_epi16(tmp);
         _mm_store_si128((__m128i*)dst, tmp2);
     }
 
     inline void batch<float, 8>::store_unaligned(int16_t* dst) const
     {
-        __m256i tmp = _mm256_cvtps_epi32(m_value);
+        __m256i tmp = _mm256_cvtps_epi32(this->m_value);
         __m128i tmp2 = detail::xsimd_cvtepi32_epi16(tmp);
         _mm_storeu_si128((__m128i*)dst, tmp2);
     }
 
     inline void batch<float, 8>::store_aligned(uint16_t* dst) const
     {
-        __m256i tmp = _mm256_cvtps_epi32(m_value);
+        __m256i tmp = _mm256_cvtps_epi32(this->m_value);
         __m128i tmp2 = detail::xsimd_cvtepi32_epu16(tmp);
         _mm_store_si128((__m128i*)dst, tmp2);
     }
 
     inline void batch<float, 8>::store_unaligned(uint16_t* dst) const
     {
-        __m256i tmp = _mm256_cvtps_epi32(m_value);
+        __m256i tmp = _mm256_cvtps_epi32(this->m_value);
         __m128i tmp2 = detail::xsimd_cvtepi32_epu16(tmp);
         _mm_storeu_si128((__m128i*)dst, tmp2);
     }
 
     inline void batch<float, 8>::store_aligned(int32_t* dst) const
     {
-        _mm256_store_si256((__m256i*)dst, _mm256_cvtps_epi32(m_value));
+        _mm256_store_si256((__m256i*)dst, _mm256_cvtps_epi32(this->m_value));
     }
 
     inline void batch<float, 8>::store_unaligned(int32_t* dst) const
     {
-        _mm256_storeu_si256((__m256i*)dst, _mm256_cvtps_epi32(m_value));
+        _mm256_storeu_si256((__m256i*)dst, _mm256_cvtps_epi32(this->m_value));
     }
 
     inline void batch<float, 8>::store_aligned(float* dst) const
     {
-        _mm256_store_ps(dst, m_value);
+        _mm256_store_ps(dst, this->m_value);
     }
 
     inline void batch<float, 8>::store_unaligned(float* dst) const
     {
-        _mm256_storeu_ps(dst, m_value);
+        _mm256_storeu_ps(dst, this->m_value);
     }
 
     inline void batch<float, 8>::store_aligned(double* dst) const
     {
         alignas(32) float tmp[8];
-        _mm256_store_ps(tmp, m_value);
+        _mm256_store_ps(tmp, this->m_value);
         dst[0] = static_cast<double>(tmp[0]);
         dst[1] = static_cast<double>(tmp[1]);
         dst[2] = static_cast<double>(tmp[2]);
@@ -450,13 +477,6 @@ namespace xsimd
         store_aligned(dst);
     }
 
-    inline float batch<float, 8>::operator[](std::size_t index) const
-    {
-        alignas(32) float x[8];
-        store_aligned(x);
-        return x[index & 7];
-    }
-
     namespace detail
     {
         template <>
@@ -634,20 +654,20 @@ namespace xsimd
                 return _mm_cvtss_f32(_mm256_extractf128_ps(tmp, 0));
             }
 
-            static batch_type haddp(const simd_batch<batch_type>* row)
+            static batch_type haddp(const batch_type* row)
             {
                 // row = (a,b,c,d,e,f,g,h)
                 // tmp0 = (a0+a1, a2+a3, b0+b1, b2+b3, a4+a5, a6+a7, b4+b5, b6+b7)
-                __m256 tmp0 = _mm256_hadd_ps(row[0](), row[1]());
+                __m256 tmp0 = _mm256_hadd_ps(row[0], row[1]);
                 // tmp1 = (c0+c1, c2+c3, d1+d2, d2+d3, c4+c5, c6+c7, d4+d5, d6+d7)
-                __m256 tmp1 = _mm256_hadd_ps(row[2](), row[3]());
+                __m256 tmp1 = _mm256_hadd_ps(row[2], row[3]);
                 // tmp1 = (a0+a1+a2+a3, b0+b1+b2+b3, c0+c1+c2+c3, d0+d1+d2+d3,
                 // a4+a5+a6+a7, b4+b5+b6+b7, c4+c5+c6+c7, d4+d5+d6+d7)
                 tmp1 = _mm256_hadd_ps(tmp0, tmp1);
                 // tmp0 = (e0+e1, e2+e3, f0+f1, f2+f3, e4+e5, e6+e7, f4+f5, f6+f7)
-                tmp0 = _mm256_hadd_ps(row[4](), row[5]());
+                tmp0 = _mm256_hadd_ps(row[4], row[5]);
                 // tmp2 = (g0+g1, g2+g3, h0+h1, h2+h3, g4+g5, g6+g7, h4+h5, h6+h7)
-                __m256 tmp2 = _mm256_hadd_ps(row[6](), row[7]());
+                __m256 tmp2 = _mm256_hadd_ps(row[6], row[7]);
                 // tmp2 = (e0+e1+e2+e3, f0+f1+f2+f3, g0+g1+g2+g3, h0+h1+h2+h3,
                 // e4+e5+e6+e7, f4+f5+f6+f7, g4+g5+g6+g7, h4+h5+h6+h7)
                 tmp2 = _mm256_hadd_ps(tmp0, tmp2);
