@@ -47,11 +47,23 @@ namespace xsimd
 
         operator __m256i() const;
 
+        bool_proxy<T> operator[](std::size_t index);
         bool operator[](std::size_t index) const;
 
+        __m256i get_value() const;
+
     private:
 
-        __m256i m_value;
+        template <class... Args>
+        batch_bool<T, N>& load_values(Args... args);
+
+        union
+        {
+            __m256i m_value;
+            T m_array[N];
+        };
+
+        friend class simd_batch_bool<batch_bool<T, N>>;
     };
 
     template <class T, std::size_t N>
@@ -91,12 +103,6 @@ namespace xsimd
         using base_type::load_unaligned;
         using base_type::store_aligned;
         using base_type::store_unaligned;
-
-        T operator[](std::size_t index) const;
-
-    protected:
-
-        __m256i m_value;
     };
 
     namespace avx_detail
@@ -124,6 +130,30 @@ namespace xsimd
         {
             return _mm256_setr_epi64x(args...);
         }
+
+        template <class T>
+        inline __m256i int_set(std::integral_constant<std::size_t, 1>, T v)
+        {
+            return _mm256_set1_epi8(v);
+        }
+
+        template <class T>
+        inline __m256i int_set(std::integral_constant<std::size_t, 2>, T v)
+        {
+            return _mm256_set1_epi16(v);
+        }
+
+        template <class T>
+        inline __m256i int_set(std::integral_constant<std::size_t, 4>, T v)
+        {
+            return _mm256_set1_epi32(v);
+        }
+
+        template <class T>
+        inline __m256i int_set(std::integral_constant<std::size_t, 8>, T v)
+        {
+            return _mm256_set1_epi64x(v);
+        }
     }
 
     /*****************************************
@@ -144,7 +174,8 @@ namespace xsimd
     template <class T, std::size_t N>
     template <class... Args, class>
     inline avx_int_batch_bool<T, N>::avx_int_batch_bool(Args... args)
-        : m_value(avx_detail::int_init(std::integral_constant<std::size_t, sizeof(T)>{}, -static_cast<T>(static_cast<bool>(args))...))
+        : m_value(avx_detail::int_init(std::integral_constant<std::size_t, sizeof(T)>{},
+                                       static_cast<T>(args ? typename std::make_signed<T>::type{-1} : 0)...))
     {
     }
 
@@ -168,13 +199,32 @@ namespace xsimd
     }
 
     template <class T, std::size_t N>
-    inline bool avx_int_batch_bool<T, N>::operator[](std::size_t index) const
+    inline bool_proxy<T> avx_int_batch_bool<T, N>::operator[](std::size_t index)
     {
-        alignas(32) T x[N];
-        _mm256_store_si256((__m256i*)x, m_value);
-        return static_cast<bool>(x[index & (N - 1)]);
+        return bool_proxy<T>(m_array[index & (N - 1)]);
     }
 
+    template <class T, std::size_t N>
+    inline bool avx_int_batch_bool<T, N>::operator[](std::size_t index) const
+    {
+        return static_cast<bool>(m_array[index & (N - 1)]);
+    }
+
+    template <class T, std::size_t N>
+    inline __m256i avx_int_batch_bool<T, N>::get_value() const
+    {
+        return m_value;
+    }
+
+    template <class T, std::size_t N>
+    template <class... Args>
+    inline batch_bool<T, N>& avx_int_batch_bool<T, N>::load_values(Args... args)
+    {
+        m_value = avx_detail::int_init(std::integral_constant<std::size_t, sizeof(T)>{},
+                                       static_cast<T>(args ? typename std::make_signed<T>::type{-1} : 0)...);
+        return (*this)();
+    }
+    
     namespace detail
     {
         template <class T, std::size_t N>
@@ -295,116 +345,104 @@ namespace xsimd
 
     template <class T, std::size_t N>
     inline avx_int_batch<T, N>::avx_int_batch(T i)
-        : m_value(sizeof(T) == 1 ? _mm256_set1_epi8(i)  :
-                  sizeof(T) == 2 ? _mm256_set1_epi16(i) :
-                  sizeof(T) == 4 ? _mm256_set1_epi32(i) : 
-                                   _mm256_set1_epi64x(i))
+        : base_type(avx_detail::int_set(std::integral_constant<std::size_t, sizeof(T)>{}, i))
     {
     }
 
     template <class T, std::size_t N>
     template <class... Args, class>
     inline avx_int_batch<T, N>::avx_int_batch(Args... args)
-        : m_value(avx_detail::int_init(std::integral_constant<std::size_t, sizeof(T)>{}, args...))
+        : base_type(avx_detail::int_init(std::integral_constant<std::size_t, sizeof(T)>{}, args...))
     {
     }
 
     template <class T, std::size_t N>
     inline avx_int_batch<T, N>::avx_int_batch(const T* src)
-        : m_value(_mm256_loadu_si256((__m256i const*)src))
+        : base_type(_mm256_loadu_si256((__m256i const*)src))
     {
     }
 
     template <class T, std::size_t N>
     inline avx_int_batch<T, N>::avx_int_batch(const T* src, aligned_mode)
-        : m_value(_mm256_load_si256((__m256i const*)src))
+        : base_type(_mm256_load_si256((__m256i const*)src))
     {
     }
 
     template <class T, std::size_t N>
     inline avx_int_batch<T, N>::avx_int_batch(const T* src, unaligned_mode)
-        : m_value(_mm256_loadu_si256((__m256i const*)src))
+        : base_type(_mm256_loadu_si256((__m256i const*)src))
     {
     }
 
     template <class T, std::size_t N>
     inline avx_int_batch<T, N>::avx_int_batch(const __m256i& rhs)
-        : m_value(rhs)
+        : base_type(rhs)
     {
     }
 
     template <class T, std::size_t N>
     inline avx_int_batch<T, N>& avx_int_batch<T, N>::operator=(const __m256i& rhs)
     {
-        m_value = rhs;
+        this->m_value = rhs;
         return *this;
     }
 
     template <class T, std::size_t N>
     inline avx_int_batch<T, N>::operator __m256i() const
     {
-        return m_value;
+        return this->m_value;
     }
 
     template <class T, std::size_t N>
     inline batch<T, N>& avx_int_batch<T, N>::load_aligned(const T* src)
     {
-        m_value = _mm256_load_si256((__m256i const*) src);
+        this->m_value = _mm256_load_si256((__m256i const*) src);
         return (*this)();
     }
 
     template <class T, std::size_t N>
     inline batch<T, N>& avx_int_batch<T, N>::load_unaligned(const T* src)
     {
-        m_value = _mm256_loadu_si256((__m256i const*) src);
+        this->m_value = _mm256_loadu_si256((__m256i const*) src);
         return (*this)();
     }
 
     template <class T, std::size_t N>
     inline batch<T, N>& avx_int_batch<T, N>::load_aligned(const flipped_sign_type_t<T>* src)
     {
-        m_value = _mm256_load_si256((__m256i const*) src);
+        this->m_value = _mm256_load_si256((__m256i const*) src);
         return (*this)();
     }
 
     template <class T, std::size_t N>
     inline batch<T, N>& avx_int_batch<T, N>::load_unaligned(const flipped_sign_type_t<T>* src)
     {
-        m_value = _mm256_loadu_si256((__m256i const*) src);
+        this->m_value = _mm256_loadu_si256((__m256i const*) src);
         return (*this)();
     }
 
     template <class T, std::size_t N>
     inline void avx_int_batch<T, N>::store_aligned(T* dst) const
     {
-        _mm256_store_si256((__m256i*) dst, m_value);
+        _mm256_store_si256((__m256i*) dst, this->m_value);
     }
 
     template <class T, std::size_t N>
     inline void avx_int_batch<T, N>::store_unaligned(T* dst) const
     {
-        _mm256_storeu_si256((__m256i*) dst, m_value);
+        _mm256_storeu_si256((__m256i*) dst, this->m_value);
     }
 
     template <class T, std::size_t N>
     inline void avx_int_batch<T, N>::store_aligned(flipped_sign_type_t<T>* dst) const
     {
-        _mm256_store_si256((__m256i*) dst, m_value);
+        _mm256_store_si256((__m256i*) dst, this->m_value);
     }
 
     template <class T, std::size_t N>
     inline void avx_int_batch<T, N>::store_unaligned(flipped_sign_type_t<T>* dst) const
     {
-        _mm256_storeu_si256((__m256i*) dst, m_value);
-    }
-
-
-    template <class T, std::size_t N>
-    inline T avx_int_batch<T, N>::operator[](std::size_t index) const
-    {
-        alignas(32) T x[N];
-        store_aligned(x);
-        return x[index & (N - 1)];
+        _mm256_storeu_si256((__m256i*) dst, this->m_value);
     }
 
     namespace detail
