@@ -68,9 +68,16 @@ namespace xsimd
         const bool& operator[](std::size_t index) const;
         bool& operator[](std::size_t index);
 
+        const std::array<bool, N>& get_value() const;
+
     private:
 
+        template <class... Args>
+        batch_bool<T, N>& load_values(Args... args);
+        
         std::array<bool, N> m_value;
+
+        friend class simd_batch_bool<batch_bool<T, N>>;
     };
 
     /***************
@@ -84,6 +91,7 @@ namespace xsimd
         static constexpr std::size_t size = N;
         using batch_bool_type = batch_bool<T, N>;
         static constexpr std::size_t align = XSIMD_DEFAULT_ALIGNMENT;
+        using storage_type = std::array<T, N>;
     };
 
     template <typename T, std::size_t N>
@@ -93,6 +101,7 @@ namespace xsimd
 
         using self_type = batch<T, N>;
         using base_type = simd_batch<self_type>;
+        using storage_type = typename base_type::storage_type;
 
         batch();
         explicit batch(T f);
@@ -104,23 +113,27 @@ namespace xsimd
         >
         batch(Args... exactly_N_scalars);
 
+        // Constructor from value_type of batch_bool
+        batch(const std::array<bool, N>& src);
+
         explicit batch(const T* src);
         batch(const T* src, aligned_mode);
         batch(const T* src, unaligned_mode);
         batch(const std::array<T, N>& rhs);
         batch& operator=(const std::array<T, N>& rhs);
+        batch& operator=(const std::array<bool, N>& rhs);
 
         operator std::array<T, N>() const;
 
-        XSIMD_DECLARE_LOAD_STORE_ALL(T, N);
+        XSIMD_DECLARE_LOAD_STORE_ALL(T, N)
 
         using base_type::load_aligned;
         using base_type::load_unaligned;
         using base_type::store_aligned;
         using base_type::store_unaligned;
 
-        const T& operator[](std::size_t index) const;
         T& operator[](std::size_t index);
+        const T& operator[](std::size_t index) const;
 
     private:
 
@@ -128,8 +141,6 @@ namespace xsimd
         batch& load_unaligned_impl(const U* src);
         template<typename U>
         void store_unaligned_impl(U* src) const;
-
-        std::array<T, N> m_value;
     };
 
     template <typename T, std::size_t N>
@@ -469,6 +480,20 @@ namespace xsimd
         return m_value[index];
     }
 
+    template <typename T, std::size_t N>
+    inline const std::array<bool, N>& batch_bool<T, N>::get_value() const
+    {
+        return m_value;
+    }
+
+    template <typename T, std::size_t N>
+    template <class... Args>
+    inline batch_bool<T, N>& batch_bool<T, N>::load_values(Args... args)
+    {
+        m_value = std::array<bool, N>({args...});
+        return *this;
+    }
+
     namespace detail
     {
         template <class T, std::size_t N>
@@ -542,17 +567,53 @@ namespace xsimd
 
     template <typename T, std::size_t N>
     inline batch<T, N>::batch(T f)
-        : m_value(detail::array_from_scalar<T, N>(f))
+        : base_type(detail::array_from_scalar<T, N>(f))
     {
     }
 
     template <typename T, std::size_t N>
     template <typename... Args, typename Enable>
     inline batch<T, N>::batch(Args... exactly_N_scalars)
-        : m_value{ static_cast<T>(exactly_N_scalars)... }
+        : base_type(storage_type{ static_cast<T>(exactly_N_scalars)... })
     {
     }
 
+    namespace detail
+    {
+        template <bool integral>
+        struct all_bits
+        {
+            template <class T>
+            static T get(T)
+            {
+                return ~T(0);
+            }
+        };
+
+        template <>
+        struct all_bits<false>
+        {
+            template <class T>
+            static T get(T)
+            {
+                T res(0);
+                using int_type = as_unsigned_integer_t<T>;
+                *reinterpret_cast<int_type*>(&res) = ~int_type(0);
+                return res;
+            }
+        };
+    }
+
+    template <typename T, std::size_t N>
+    inline batch<T, N>::batch(const std::array<bool, N>& src)
+    {
+        using all_bits = detail::all_bits<std::is_integral<T>::value>;
+        for(std::size_t i = 0; i < N; ++i)
+        {
+            this->m_value[i] = src[i] ? all_bits::get(T(0)) : T(0);
+        }
+    }
+
     template <typename T, std::size_t N>
     inline batch<T, N>::batch(const T* src)
         : batch(src, unaligned_mode())
@@ -567,27 +628,36 @@ namespace xsimd
 
     template <typename T, std::size_t N>
     inline batch<T, N>::batch(const T* src, unaligned_mode)
-        : m_value(detail::array_from_pointer<T, N>(src))
+        : base_type(detail::array_from_pointer<T, N>(src))
     {
     }
 
     template <typename T, std::size_t N>
     inline batch<T, N>::batch(const std::array<T, N>& rhs)
-        : m_value(rhs)
+        : base_type(rhs)
     {
     }
 
     template <typename T, std::size_t N>
     inline batch<T, N>& batch<T, N>::operator=(const std::array<T, N>& rhs)
     {
-        m_value = rhs;
+        this->m_value = rhs;
+        return *this;
+    }
+
+    template <typename T, std::size_t N>
+    inline batch<T, N>& batch<T, N>::operator=(const std::array<bool, N>& rhs)
+    {
+        using all_bits = detail::all_bits<std::is_integral<T>::value>;
+        std::transform(rhs.cbegin(), rhs.cend(), this->m_value.begin(),
+                       [](bool b) -> T { return b ? all_bits::get(T(0)) : T(0); });
         return *this;
     }
 
     template <typename T, std::size_t N>
     inline batch<T, N>::operator std::array<T, N>() const
     {
-        return m_value;
+        return this->m_value;
     }
 
 #define FALLBACK_DEFINE_LOAD_STORE(TYPE)                             \
@@ -625,24 +695,13 @@ namespace xsimd
 
 #undef FALLBACK_DEFINE_LOAD_STORE
 
-    template <typename T, std::size_t N>
-    inline const T& batch<T, N>::operator[](std::size_t index) const
-    {
-        return m_value[index];
-    }
-
-    template <typename T, std::size_t N>
-    inline T& batch<T, N>::operator[](std::size_t index)
-    {
-        return m_value[index];
-    }
-
     template <typename T, std::size_t N>
     template <typename U>
     inline batch<T, N>& batch<T, N>::load_unaligned_impl(const U* src)
     {
-        for(std::size_t i = 0; i < N; ++i) {
-            m_value[i] = static_cast<T>(src[i]);
+        for(std::size_t i = 0; i < N; ++i)
+        {
+            this->m_value[i] = static_cast<T>(src[i]);
         }
         return *this;
     }
@@ -651,11 +710,24 @@ namespace xsimd
     template <typename U>
     inline void batch<T, N>::store_unaligned_impl(U* dst) const
     {
-        for(std::size_t i = 0; i < N; ++i) {
-            dst[i] = static_cast<U>(m_value[i]);
+        for(std::size_t i = 0; i < N; ++i)
+        {
+            dst[i] = static_cast<U>(this->m_value[i]);
         }
     }
 
+    template <typename T, std::size_t N>
+    inline T& batch<T, N>::operator[](std::size_t index)
+    {
+        return this->m_value[index % base_type::size];
+    }
+
+    template <typename T, std::size_t N>
+    inline const T& batch<T, N>::operator[](std::size_t index) const
+    {
+        return this->m_value[index % base_type::size];
+    }
+
     namespace detail
     {
         template <class T, std::size_t N>
@@ -814,9 +886,9 @@ namespace xsimd
                 return result;
             }
 
-            static batch_type haddp(const simd_batch<batch_type>* row)
+            static batch_type haddp(const batch_type* row)
             {
-                XSIMD_FALLBACK_MAPPING_LOOP(batch, hadd(row[i]()))
+                XSIMD_FALLBACK_MAPPING_LOOP(batch, hadd(row[i]))
             }
 
             static batch_type select(const batch_bool_type& cond, const batch_type& a, const batch_type& b)
