@@ -13,53 +13,101 @@
 
 #define EXPAND(...) __VA_ARGS__
 
-#define CASE(i, op)              \
-    case i: return op(lhs, i);   \
+#define CASE(op, i)                   \
+    case i: return op(lhs, i);
 
-#define REPEAT_8(op, addx)       \
-    CASE( 1 + addx, EXPAND(op)); \
-    CASE( 2 + addx, EXPAND(op)); \
-    CASE( 3 + addx, EXPAND(op)); \
-    CASE( 4 + addx, EXPAND(op)); \
-    CASE( 5 + addx, EXPAND(op)); \
-    CASE( 6 + addx, EXPAND(op)); \
-    CASE( 7 + addx, EXPAND(op)); \
+#define XSIMD_REPEAT_8_0(op, addx)    \
+    CASE(EXPAND(op), 1 + addx);       \
+    CASE(EXPAND(op), 2 + addx);       \
+    CASE(EXPAND(op), 3 + addx);       \
+    CASE(EXPAND(op), 4 + addx);       \
+    CASE(EXPAND(op), 5 + addx);       \
+    CASE(EXPAND(op), 6 + addx);       \
+    CASE(EXPAND(op), 7 + addx);
 
-#define REPEAT_32(op, addx)      \
-    CASE( 1 + addx, EXPAND(op)); \
-    CASE( 2 + addx, EXPAND(op)); \
-    CASE( 3 + addx, EXPAND(op)); \
-    CASE( 4 + addx, EXPAND(op)); \
-    CASE( 5 + addx, EXPAND(op)); \
-    CASE( 6 + addx, EXPAND(op)); \
-    CASE( 7 + addx, EXPAND(op)); \
-    CASE( 8 + addx, EXPAND(op)); \
-    CASE( 9 + addx, EXPAND(op)); \
-    CASE(10 + addx, EXPAND(op)); \
-    CASE(11 + addx, EXPAND(op)); \
-    CASE(12 + addx, EXPAND(op)); \
-    CASE(13 + addx, EXPAND(op)); \
-    CASE(14 + addx, EXPAND(op)); \
-    CASE(15 + addx, EXPAND(op)); \
-    CASE(16 + addx, EXPAND(op)); \
-    CASE(17 + addx, EXPAND(op)); \
-    CASE(18 + addx, EXPAND(op)); \
-    CASE(19 + addx, EXPAND(op)); \
-    CASE(20 + addx, EXPAND(op)); \
-    CASE(21 + addx, EXPAND(op)); \
-    CASE(22 + addx, EXPAND(op)); \
-    CASE(23 + addx, EXPAND(op)); \
-    CASE(24 + addx, EXPAND(op)); \
-    CASE(25 + addx, EXPAND(op)); \
-    CASE(26 + addx, EXPAND(op)); \
-    CASE(27 + addx, EXPAND(op)); \
-    CASE(28 + addx, EXPAND(op)); \
-    CASE(29 + addx, EXPAND(op)); \
-    CASE(30 + addx, EXPAND(op)); \
-    CASE(31 + addx, EXPAND(op)); \
+#define XSIMD_REPEAT_8_N(op, addx)    \
+    CASE(EXPAND(op), 0 + addx);       \
+    XSIMD_REPEAT_8_0(op, addx);
+
+#define XSIMD_REPEAT_8(op)            \
+    XSIMD_REPEAT_8_0(op, 0);
+
+#define XSIMD_REPEAT_16_0(op, addx)   \
+    XSIMD_REPEAT_8_0(op, 0 + addx);   \
+    XSIMD_REPEAT_8_N(op, 8 + addx);
+
+#define XSIMD_REPEAT_16_N(op, addx)   \
+    XSIMD_REPEAT_8_N(op, 0 + addx);   \
+    XSIMD_REPEAT_8_N(op, 8 + addx);
+
+#define XSIMD_REPEAT_16(op)           \
+    XSIMD_REPEAT_16_0(op, 0);
+
+#define XSIMD_REPEAT_32_0(op, addx)   \
+    XSIMD_REPEAT_16_0(op, 0 + addx);  \
+    XSIMD_REPEAT_16_N(op, 16 + addx);
+
+#define XSIMD_REPEAT_32_N(op, addx)   \
+    XSIMD_REPEAT_16_N(op, 0 + addx);  \
+    XSIMD_REPEAT_16_N(op, 16 + addx);
+
+#define XSIMD_REPEAT_32(op)           \
+    XSIMD_REPEAT_32_0(op, 0);
+
+#define XSIMD_REPEAT_64(op)           \
+    XSIMD_REPEAT_32_0(op, 0);         \
+    XSIMD_REPEAT_32_N(op, 32);
 
 namespace xsimd
 {
+    /********************
+     * batch_bool<T, 8> *
+     ********************/
+
+    template <class T>
+    struct simd_batch_traits<batch_bool<T, 8>>
+    {
+        using value_type = T;
+        static constexpr std::size_t size = 8;
+        using batch_type = batch<T, 8>;
+        static constexpr std::size_t align = XSIMD_DEFAULT_ALIGNMENT;
+    };
+
+    template <class T>
+    class batch_bool<T, 8> : public simd_batch_bool<batch_bool<T, 8>>
+    {
+    public:
+
+        using simd_type = uint16x8_t;
+
+        batch_bool();
+        explicit batch_bool(bool b);
+        batch_bool(bool b0, bool b1, bool b2, bool b3,
+                   bool b4, bool b5, bool b6, bool b7);
+        batch_bool(const simd_type& rhs);
+        template <class V>
+        batch_bool(const batch_bool<V, 8>& rhs);
+
+        batch_bool& operator=(const simd_type& rhs);
+
+        operator simd_type() const;
+
+        bool_proxy<uint16_t> operator[](std::size_t index);
+        bool operator[](std::size_t index) const;
+
+    private:
+
+        batch_bool<T, 8>& load_values(bool b0, bool b1, bool b2, bool b3,
+                                      bool b4, bool b5, bool b6, bool b7);
+        union
+        {
+            simd_type m_value;
+            uint16_t m_array[8];
+        };
+
+        friend class simd_batch_bool<batch_bool<T, 8>>;
+    };
+
     /********************
      * batch_bool<T, 4> *
      ********************/
@@ -91,11 +139,20 @@ namespace xsimd
 
         operator simd_type() const;
 
+        bool_proxy<uint32_t> operator[](std::size_t index);
         bool operator[](std::size_t index) const;
 
     private:
 
-        simd_type m_value;
+        batch_bool<T, 4>& load_values(bool b0, bool b1, bool b2, bool b3);
+
+        union
+        {
+            simd_type m_value;
+            uint32_t m_array[4];
+        };
+
+        friend class simd_batch_bool<batch_bool<T, 4>>;
     };
 
     /********************
@@ -129,11 +186,20 @@ namespace xsimd
 
         operator simd_type() const;
 
+        bool_proxy<uint64_t> operator[](std::size_t index);
         bool operator[](std::size_t index) const;
 
     private:
 
-        simd_type m_value;
+        batch_bool<T, 2>& load_values(bool b0, bool b1);
+
+        union
+        {
+            simd_type m_value;
+            uint64_t m_array[2];
+        };
+
+        friend class simd_batch_bool<batch_bool<T, 2>>;
     };
 
     /*********************
@@ -158,9 +224,9 @@ namespace xsimd
 
         batch_bool();
         explicit batch_bool(bool b);
-        batch_bool( bool d1,  bool d2,  bool d3,  bool d4,
-                    bool d5,  bool d6,  bool d7,  bool d8,
-                    bool d9, bool d10, bool d11, bool d12,
+        batch_bool(bool d1,  bool d2,  bool d3,  bool d4,
+                   bool d5,  bool d6,  bool d7,  bool d8,
+                   bool d9, bool d10, bool d11, bool d12,
                    bool d13, bool d14, bool d15, bool d16);
         batch_bool(const simd_type& rhs);
         template <class V>
@@ -170,13 +236,166 @@ namespace xsimd
 
         operator simd_type() const;
 
+        bool_proxy<uint8_t> operator[](std::size_t index);
         bool operator[](std::size_t index) const;
 
     private:
 
-        simd_type m_value;
+        batch_bool<T, 16>& load_values(bool d1,  bool d2,  bool d3,  bool d4,
+                                       bool d5,  bool d6,  bool d7,  bool d8,
+                                       bool d9, bool d10, bool d11, bool d12,
+                                       bool d13, bool d14, bool d15, bool d16);
+
+        union
+        {
+            simd_type m_value;
+            uint8_t m_array[16];
+        };
+
+        friend class simd_batch_bool<batch_bool<T, 16>>;
     };
 
+    /***********************************
+     * batch_bool<T, 8> implementation *
+     ***********************************/
+
+    template <class T>
+    inline batch_bool<T, 8>::batch_bool()
+    {
+    }
+
+    template <class T>
+    inline batch_bool<T, 8>::batch_bool(bool b)
+        : m_value(vdupq_n_u16(
+            static_cast<uint16_t>(-(int)b))
+        )
+    {
+    }
+
+    template <class T>
+    inline batch_bool<T, 8>::batch_bool(bool b0, bool b1, bool b2, bool b3,
+                                        bool b4, bool b5, bool b6, bool b7)
+        : m_value{
+            static_cast<uint16_t>(-int(b0)),
+            static_cast<uint16_t>(-int(b1)),
+            static_cast<uint16_t>(-int(b2)),
+            static_cast<uint16_t>(-int(b3)),
+            static_cast<uint16_t>(-int(b4)),
+            static_cast<uint16_t>(-int(b5)),
+            static_cast<uint16_t>(-int(b6)),
+            static_cast<uint16_t>(-int(b7))}
+    {
+    }
+
+    template <class T>
+    inline batch_bool<T, 8>::batch_bool(const simd_type& rhs)
+        : m_value(rhs)
+    {
+    }
+
+    template <class T>
+    template <class V>
+    inline batch_bool<T, 8>::batch_bool(const batch_bool<V, 8>& rhs)
+        : m_value(static_cast<simd_type>(rhs))
+    {
+    }
+
+    template <class T>
+    inline batch_bool<T, 8>& batch_bool<T, 8>::operator=(const simd_type& rhs)
+    {
+        m_value = rhs;
+        return *this;
+    }
+
+    template <class T>
+    inline batch_bool<T, 8>::operator uint16x8_t() const
+    {
+        return m_value;
+    }
+
+    template <class T>
+    inline bool_proxy<uint16_t> batch_bool<T, 8>::operator[](std::size_t index)
+    {
+        return bool_proxy<uint16_t>(m_array[index & 7]);
+    }
+
+    template <class T>
+    inline bool batch_bool<T, 8>::operator[](std::size_t index) const
+    {
+        return static_cast<bool>(m_array[index & 7]);
+    }
+
+    template <class T>
+    inline batch_bool<T, 8>& batch_bool<T, 8>::load_values(bool b0, bool b1, bool b2, bool b3,
+                                                           bool b4, bool b5, bool b6, bool b7)
+    {
+        m_value[0] = static_cast<uint16_t>(-int(b0));
+        m_value[1] = static_cast<uint16_t>(-int(b1));
+        m_value[2] = static_cast<uint16_t>(-int(b2));
+        m_value[3] = static_cast<uint16_t>(-int(b3));
+        m_value[4] = static_cast<uint16_t>(-int(b4));
+        m_value[5] = static_cast<uint16_t>(-int(b5));
+        m_value[6] = static_cast<uint16_t>(-int(b6));
+        m_value[7] = static_cast<uint16_t>(-int(b7));
+        return *this;
+    }
+
+    namespace detail
+    {
+        template <class T>
+        struct batch_bool_kernel<T, 8>
+        {
+            using batch_type = batch_bool<T, 8>;
+
+            static batch_type bitwise_and(const batch_type& lhs, const batch_type& rhs)
+            {
+                return vandq_u16(lhs, rhs);
+            }
+
+            static batch_type bitwise_or(const batch_type& lhs, const batch_type& rhs)
+            {
+                return vorrq_u16(lhs, rhs);
+            }
+
+            static batch_type bitwise_xor(const batch_type& lhs, const batch_type& rhs)
+            {
+                return veorq_u16(lhs, rhs);
+            }
+
+            static batch_type bitwise_not(const batch_type& rhs)
+            {
+                return vmvnq_u16(rhs);
+            }
+
+            static batch_type bitwise_andnot(const batch_type& lhs, const batch_type& rhs)
+            {
+                return vbicq_u16(lhs, rhs);
+            }
+
+            static batch_type equal(const batch_type& lhs, const batch_type& rhs)
+            {
+                return vceqq_u16(lhs, rhs);
+            }
+
+            static batch_type not_equal(const batch_type& lhs, const batch_type& rhs)
+            {
+                return veorq_u16(lhs, rhs);
+            }
+
+            static bool all(const batch_type& rhs)
+            {
+                uint16x4_t tmp = vand_u16(vget_low_u16(rhs), vget_high_u16(rhs));
+                return vget_lane_u16(vpmin_u16(tmp, tmp), 0) != 0;
+            }
+
+            static bool any(const batch_type& rhs)
+            {
+                uint16x4_t tmp = vorr_u16(vget_low_u16(rhs), vget_high_u16(rhs));
+                return vget_lane_u16(vpmax_u16(tmp, tmp), 0);
+            }
+        };
+    }
+
     /***********************************
      * batch_bool<T, 4> implementation *
      ***********************************/
@@ -195,12 +414,12 @@ namespace xsimd
     }
 
     template <class T>
-    inline batch_bool<T, 4>::batch_bool(bool b1, bool b2, bool b3, bool b4)
+    inline batch_bool<T, 4>::batch_bool(bool b0, bool b1, bool b2, bool b3)
         : m_value{
+            static_cast<uint32_t>(-int(b0)),
             static_cast<uint32_t>(-int(b1)),
             static_cast<uint32_t>(-int(b2)),
-            static_cast<uint32_t>(-int(b3)),
-            static_cast<uint32_t>(-int(b4))}
+            static_cast<uint32_t>(-int(b3))}
     {
     }
 
@@ -230,10 +449,26 @@ namespace xsimd
         return m_value;
     }
 
+    template <class T>
+    inline bool_proxy<uint32_t> batch_bool<T, 4>::operator[](std::size_t index)
+    {
+        return bool_proxy<uint32_t>(m_array[index & 3]);
+    }
+
     template <class T>
     inline bool batch_bool<T, 4>::operator[](std::size_t index) const
     {
-        return static_cast<bool>(m_value[index]);
+        return static_cast<bool>(m_array[index & 3]);
+    }
+
+    template <class T>
+    inline batch_bool<T, 4>& batch_bool<T, 4>::load_values(bool b0, bool b1, bool b2, bool b3)
+    {
+        m_value[0] = static_cast<uint32_t>(-int(b0));
+        m_value[1] = static_cast<uint32_t>(-int(b1));
+        m_value[2] = static_cast<uint32_t>(-int(b2));
+        m_value[3] = static_cast<uint32_t>(-int(b3));
+        return *this;
     }
 
     namespace detail
@@ -310,9 +545,9 @@ namespace xsimd
     }
 
     template <class T>
-    inline batch_bool<T, 16>::batch_bool( bool b1,  bool b2,  bool b3,  bool b4,
-                                          bool b5,  bool b6,  bool b7,  bool b8,
-                                          bool b9, bool b10, bool b11, bool b12,
+    inline batch_bool<T, 16>::batch_bool(bool b1,  bool b2,  bool b3,  bool b4,
+                                         bool b5,  bool b6,  bool b7,  bool b8,
+                                         bool b9, bool b10, bool b11, bool b12,
                                          bool b13, bool b14, bool b15, bool b16)
         : m_value{
             static_cast<uint8_t>(-int(b1)),
@@ -360,10 +595,41 @@ namespace xsimd
         return m_value;
     }
 
+    template <class T>
+    inline bool_proxy<uint8_t> batch_bool<T, 16>::operator[](std::size_t index)
+    {
+        return bool_proxy<uint8_t>(m_array[index & 15]);
+    }
+
     template <class T>
     inline bool batch_bool<T, 16>::operator[](std::size_t index) const
     {
-        return static_cast<bool>(m_value[index]);
+        return static_cast<bool>(m_array[index & 15]);
+    }
+
+    template <class T>
+    inline batch_bool<T, 16>& batch_bool<T, 16>::load_values(bool b1,  bool b2,  bool b3,  bool b4,
+                                                             bool b5,  bool b6,  bool b7,  bool b8,
+                                                             bool b9,  bool b10, bool b11, bool b12,
+                                                             bool b13, bool b14, bool b15, bool b16)
+    {
+        m_value[0] = static_cast<uint8_t>(-int(b1));
+        m_value[1] = static_cast<uint8_t>(-int(b2));
+        m_value[2] = static_cast<uint8_t>(-int(b3));
+        m_value[3] = static_cast<uint8_t>(-int(b4));
+        m_value[4] = static_cast<uint8_t>(-int(b5));
+        m_value[5] = static_cast<uint8_t>(-int(b6));
+        m_value[6] = static_cast<uint8_t>(-int(b7));
+        m_value[7] = static_cast<uint8_t>(-int(b8));
+        m_value[8] = static_cast<uint8_t>(-int(b9));
+        m_value[9] = static_cast<uint8_t>(-int(b10));
+        m_value[10] = static_cast<uint8_t>(-int(b11));
+        m_value[11] = static_cast<uint8_t>(-int(b12));
+        m_value[12] = static_cast<uint8_t>(-int(b13));
+        m_value[13] = static_cast<uint8_t>(-int(b14));
+        m_value[14] = static_cast<uint8_t>(-int(b15));
+        m_value[15] = static_cast<uint8_t>(-int(b16));
+        return *this;
     }
 
     namespace detail
@@ -438,10 +704,10 @@ namespace xsimd
     }
 
     template <class T>
-    inline batch_bool<T, 2>::batch_bool(bool b1, bool b2)
+    inline batch_bool<T, 2>::batch_bool(bool b0, bool b1)
         : m_value{
-            static_cast<uint64_t>(-int(b1)),
-            static_cast<uint64_t>(-int(b2))}
+            static_cast<uint64_t>(-int(b0)),
+            static_cast<uint64_t>(-int(b1))}
     {
     }
 
@@ -471,10 +737,24 @@ namespace xsimd
         return m_value;
     }
 
+    template <class T>
+    inline bool_proxy<uint64_t> batch_bool<T, 2>::operator[](std::size_t index)
+    {
+        return bool_proxy<uint64_t>(m_array[index & 1]);
+    }
+
     template <class T>
     inline bool batch_bool<T, 2>::operator[](std::size_t index) const
     {
-        return static_cast<bool>(m_value[index]);
+        return static_cast<bool>(m_array[index & 1]);
+    }
+
+    template <class T>
+    inline batch_bool<T, 2>& batch_bool<T, 2>::load_values(bool b0, bool b1)
+    {
+        m_value[0] = static_cast<uint64_t>(-int(b0));
+        m_value[1] = static_cast<uint64_t>(-int(b1));
+        return *this;
     }
 
     namespace detail
