@@ -34,11 +34,23 @@ namespace xsimd
 
         operator __m128i() const;
 
+        bool_proxy<T> operator[](std::size_t index);
         bool operator[](std::size_t index) const;
 
+        __m128i get_value() const;
+
     private:
 
-        __m128i m_value;
+        template <class... Args>
+        batch_bool<T, N>& load_values(Args... args);
+
+        union
+        {
+            __m128i m_value;
+            T m_array[N];
+        };
+
+        friend class simd_batch_bool<batch_bool<T, N>>;
     };
 
     /***********************
@@ -80,12 +92,6 @@ namespace xsimd
         using base_type::load_unaligned;
         using base_type::store_aligned;
         using base_type::store_unaligned;
-
-        T operator[](std::size_t index) const;
-
-    protected:
-
-        __m128i m_value;
     };
 
     /********************
@@ -118,6 +124,30 @@ namespace xsimd
             return _mm_set_epi64x(i1, i0);
         }
 
+        template <class T>
+        inline __m128i int_set(std::integral_constant<std::size_t, 1>, T v)
+        {
+            return _mm_set1_epi8(v);
+        }
+
+        template <class T>
+        inline __m128i int_set(std::integral_constant<std::size_t, 2>, T v)
+        {
+            return _mm_set1_epi16(v);
+        }
+
+        template <class T>
+        inline __m128i int_set(std::integral_constant<std::size_t, 4>, T v)
+        {
+            return _mm_set1_epi32(v);
+        }
+
+        template <class T>
+        inline __m128i int_set(std::integral_constant<std::size_t, 8>, T v)
+        {
+            return _mm_set1_epi64x(v);
+        }
+
         inline __m128i cmpeq_epi64_sse2(__m128i lhs, __m128i rhs)
         {
             __m128i tmp1 = _mm_cmpeq_epi32(lhs, rhs);
@@ -146,7 +176,8 @@ namespace xsimd
     template <class T, std::size_t N>
     template <class... Args, class>
     inline sse_int_batch_bool<T, N>::sse_int_batch_bool(Args... args)
-        : m_value(sse_detail::int_init(std::integral_constant<std::size_t, sizeof(T)>{}, -static_cast<T>(static_cast<bool>(args))...))
+        : m_value(sse_detail::int_init(std::integral_constant<std::size_t, sizeof(T)>{},
+                                       static_cast<T>(args ? typename std::make_signed<T>::type{-1} : 0)...))
     {
     }
 
@@ -170,13 +201,32 @@ namespace xsimd
     }
 
     template <class T, std::size_t N>
-    inline bool sse_int_batch_bool<T, N>::operator[](std::size_t index) const
+    inline bool_proxy<T> sse_int_batch_bool<T, N>::operator[](std::size_t index)
     {
-        alignas(16) T x[N];
-        _mm_store_si128((__m128i*)x, m_value);
-        return static_cast<bool>(x[index & (N - 1)]);
+        return bool_proxy<T>(m_array[index & (N - 1)]);
     }
 
+    template <class T, std::size_t N>
+    inline bool sse_int_batch_bool<T, N>::operator[](std::size_t index) const
+    {
+        return static_cast<bool>(m_array[index & (N - 1)]);
+    }
+
+    template <class T, std::size_t N>
+    inline __m128i sse_int_batch_bool<T, N>::get_value() const
+    {
+        return m_value;
+    }
+    
+    template <class T, std::size_t N>
+    template <class... Args>
+    inline batch_bool<T, N>& sse_int_batch_bool<T, N>::load_values(Args... args)
+    {
+        m_value = sse_detail::int_init(std::integral_constant<std::size_t, sizeof(T)>{},
+                                       static_cast<T>(args ? typename std::make_signed<T>::type{-1} : 0)...);
+        return (*this)();
+    }
+    
     namespace detail
     {
         template <class T>
@@ -262,115 +312,104 @@ namespace xsimd
 
     template <class T, std::size_t N>
     inline sse_int_batch<T, N>::sse_int_batch(T i)
-        : m_value(sizeof(T) == 1 ? _mm_set1_epi8(i)  :
-                  sizeof(T) == 2 ? _mm_set1_epi16(i) :
-                  sizeof(T) == 4 ? _mm_set1_epi32(i) : 
-                                   _mm_set1_epi64x(i))
+        : base_type(sse_detail::int_set(std::integral_constant<std::size_t, sizeof(T)>{}, i))
     {
     }
 
     template <class T, std::size_t N>
     template <class... Args, class>
     inline sse_int_batch<T, N>::sse_int_batch(Args... args)
-        : m_value(sse_detail::int_init(std::integral_constant<std::size_t, sizeof(T)>{}, args...))
+        : base_type(sse_detail::int_init(std::integral_constant<std::size_t, sizeof(T)>{}, args...))
     {
     }
 
     template <class T, std::size_t N>
     inline sse_int_batch<T, N>::sse_int_batch(const T* src)
-        : m_value(_mm_loadu_si128((__m128i const*)src))
+        : base_type(_mm_loadu_si128((__m128i const*)src))
     {
     }
 
     template <class T, std::size_t N>
     inline sse_int_batch<T, N>::sse_int_batch(const T* src, aligned_mode)
-        : m_value(_mm_load_si128((__m128i const*)src))
+        : base_type(_mm_load_si128((__m128i const*)src))
     {
     }
 
     template <class T, std::size_t N>
     inline sse_int_batch<T, N>::sse_int_batch(const T* src, unaligned_mode)
-        : m_value(_mm_loadu_si128((__m128i const*)src))
+        : base_type(_mm_loadu_si128((__m128i const*)src))
     {
     }
 
     template <class T, std::size_t N>
     inline sse_int_batch<T, N>::sse_int_batch(const __m128i& rhs)
-        : m_value(rhs)
+        : base_type(rhs)
     {
     }
 
     template <class T, std::size_t N>
     inline sse_int_batch<T, N>& sse_int_batch<T, N>::operator=(const __m128i& rhs)
     {
-        m_value = rhs;
+        this->m_value = rhs;
         return *this;
     }
 
     template <class T, std::size_t N>
     inline sse_int_batch<T, N>::operator __m128i() const
     {
-        return m_value;
+        return this->m_value;
     }
 
     template <class T, std::size_t N>
     inline batch<T, N>& sse_int_batch<T, N>::load_aligned(const T* src)
     {
-        m_value = _mm_load_si128((__m128i const*)src);
+        this->m_value = _mm_load_si128((__m128i const*)src);
         return (*this)();
     }
 
     template <class T, std::size_t N>
     inline batch<T, N>& sse_int_batch<T, N>::load_unaligned(const T* src)
     {
-        m_value = _mm_loadu_si128((__m128i const*)src);
+        this->m_value = _mm_loadu_si128((__m128i const*)src);
         return (*this)();
     }
 
     template <class T, std::size_t N>
     inline batch<T, N>& sse_int_batch<T, N>::load_aligned(const flipped_sign_type_t<T>* src)
     {
-        m_value = _mm_load_si128((__m128i const*)src);
+        this->m_value = _mm_load_si128((__m128i const*)src);
         return (*this)();
     }
 
     template <class T, std::size_t N>
     inline batch<T, N>& sse_int_batch<T, N>::load_unaligned(const flipped_sign_type_t<T>* src)
     {
-        m_value = _mm_loadu_si128((__m128i const*)src);
+        this->m_value = _mm_loadu_si128((__m128i const*)src);
         return (*this)();
     }
 
     template <class T, std::size_t N>
     inline void sse_int_batch<T, N>::store_aligned(T* dst) const
     {
-        _mm_store_si128((__m128i*)dst, m_value);
+        _mm_store_si128((__m128i*)dst, this->m_value);
     }
 
     template <class T, std::size_t N>
     inline void sse_int_batch<T, N>::store_unaligned(T* dst) const
     {
-        _mm_storeu_si128((__m128i*)dst, m_value);
+        _mm_storeu_si128((__m128i*)dst, this->m_value);
     }
 
     template <class T, std::size_t N>
     inline void sse_int_batch<T, N>::store_aligned(flipped_sign_type_t<T>* dst) const
     {
-        _mm_store_si128((__m128i*)dst, m_value);
+        _mm_store_si128((__m128i*)dst, this->m_value);
     }
 
     template <class T, std::size_t N>
     inline void sse_int_batch<T, N>::store_unaligned(flipped_sign_type_t<T>* dst) const
     {
-        _mm_storeu_si128((__m128i*)dst, m_value);
-    }
-
-    template <class T, std::size_t N>
-    inline T sse_int_batch<T, N>::operator[](std::size_t index) const
-    {
-        alignas(16) T x[N];
-        store_aligned(x);
-        return x[index & (N - 1)];
+        _mm_storeu_si128((__m128i*)dst, this->m_value);
     }
 
     namespace detail
