@@ -58,11 +58,23 @@
 #include "xsimd_neon_float.hpp"
 #include "xsimd_neon_int8.hpp"
 #include "xsimd_neon_uint8.hpp"
+#include "xsimd_neon_int16.hpp"
+#include "xsimd_neon_uint16.hpp"
 #include "xsimd_neon_int32.hpp"
+#include "xsimd_neon_uint32.hpp"
 #include "xsimd_neon_int64.hpp"
+#include "xsimd_neon_uint64.hpp"
 #include "xsimd_neon_complex.hpp"
 #endif
 
+#if !defined(XSIMD_INSTR_SET_AVAILABLE)
+	#if defined(XSIMD_ENABLE_FALLBACK)
+		#warning "No SIMD instructions set detected, using fallback mode."
+	#else
+		#warning "No SIMD instructions set detected, please enable SIMD instructions or activate the fallback mode. (e.g. for x86 -march=native or for ARM -mfpu=neon)"
+	#endif
+#endif
+
 #include "xsimd_utils.hpp"
 
 #endif
